`timescale 1ns/1ps

module round_pipeline
#(
	parameter DATA_WIDTH = 32
)
(
	input 				           clk_i		,
	input 				           reset_ni	 	,
	input 						   enable_i		,
	input      [DATA_WIDTH-1:0]    R0_i			,
	input      [DATA_WIDTH-1:0]    R1_i			,
	input      [DATA_WIDTH-1:0]    R2_i			,
	input      [DATA_WIDTH-1:0]    R3_i			,
	input      [DATA_WIDTH-1:0]    K_key_even	,
	input      [DATA_WIDTH-1:0]    K_key_odd	,
	
	output 	   [DATA_WIDTH-1:0]	   R0_o			,
	output 	   [DATA_WIDTH-1:0]	   R1_o			,
	output 	   [DATA_WIDTH-1:0]    R2_o			,
	output 	   [DATA_WIDTH-1:0]    R3_o			
);

wire	[DATA_WIDTH-1:0] F0;
wire	[DATA_WIDTH-1:0] F1;
wire 	[DATA_WIDTH-1:0] R3_rol1;

wire 	[DATA_WIDTH-1:0] R2_xor_F0;

function_f_singleHinst
#(
	.DATA_WIDTH(DATA_WIDTH)
)

f_INST
(
	.clk_i		   (clk_i	      ),
	.reset_ni	   (reset_ni      ),
    .R0_i		   (R0_i		  ),
    .R1_i		   (R1_i		  ),
	.S0_i          (32'b0	      ),
	.S1_i          (32'b0	      ),
	.K_key_even_i  (K_key_even    ),
	.K_key_odd_i   (K_key_odd	  ),
	
    .F0_o		   (F0			  ),
    .F1_o		   (F1			  )
);

assign R2_xor_F0 = R2_i ^ F0;
assign R3_rol1 	 = {R3_i[DATA_WIDTH-2:0],R3_i[DATA_WIDTH-1]};

assign R0_o = (enable_i) ? ({R2_xor_F0[0],R2_xor_F0[DATA_WIDTH-1:1]})  : 'b0;
assign R1_o = (enable_i) ? (R3_rol1 ^ F1							)  : 'b0;
assign R2_o = (enable_i) ? (R0_i									)  : 'b0;	
assign R3_o = (enable_i) ? (R1_i									)  : 'b0;

endmodule