`timescale 1ns/1ps

module twofishTB();

parameter DATA_WIDTH = 128;

reg clk_i;
reg reset_ni;
reg enable_i;

reg [DATA_WIDTH/4-1:0] plaintext3_i;
reg [DATA_WIDTH/4-1:0] plaintext2_i;
reg [DATA_WIDTH/4-1:0] plaintext1_i;
reg [DATA_WIDTH/4-1:0] plaintext0_i;

reg [DATA_WIDTH/4-1:0] globalkey3_i;
reg [DATA_WIDTH/4-1:0] globalkey2_i;
reg [DATA_WIDTH/4-1:0] globalkey1_i;
reg [DATA_WIDTH/4-1:0] globalkey0_i;

wire [DATA_WIDTH/4-1:0] cryptotext3_o;
wire [DATA_WIDTH/4-1:0] cryptotext2_o;
wire [DATA_WIDTH/4-1:0] cryptotext1_o;
wire [DATA_WIDTH/4-1:0] cryptotext0_o;

wire busy_o;

twofish_pipeline twofishDUT
(
	.clk_i(clk_i),
	.reset_ni(reset_ni),
	.enable_i(enable_i),
	
	.plaintext3_i(plaintext3_i),
	.plaintext2_i(plaintext2_i),
	.plaintext1_i(plaintext1_i),
	.plaintext0_i(plaintext0_i),
	
	.globalkey3_i(globalkey3_i),
	.globalkey2_i(globalkey2_i),
	.globalkey1_i(globalkey1_i),
	.globalkey0_i(globalkey0_i),
	
	.cryptotext3_o(cryptotext3_o),
	.cryptotext2_o(cryptotext2_o),
	.cryptotext1_o(cryptotext1_o),
	.cryptotext0_o(cryptotext0_o),
	
	.busy_o(busy_o)
);

always begin
	#5 clk_i <= ~clk_i;
end

initial begin
	clk_i = 1'b0;
	reset_ni = 1'b1;
	enable_i = 1'b0;
	plaintext3_i ='bx;
	plaintext2_i ='bx;
	plaintext1_i ='bx;
	plaintext0_i ='bx;
	globalkey3_i ='bx;
	globalkey2_i ='bx;
	globalkey1_i ='bx;
	globalkey0_i ='bx;
	@(posedge clk_i);
	plaintext3_i ={16'h0000,16'h0000};
	plaintext2_i ={16'h0000,16'h0000};
	plaintext1_i ={16'h0000,16'h0000};
	plaintext0_i ={16'h0000,16'h0000};
	globalkey3_i ='b0;
	globalkey2_i ='b0;
	globalkey1_i ='b0;
	globalkey0_i ='b0;
	@(posedge clk_i);
	reset_ni = 1'b0;
	@(posedge clk_i);
	reset_ni = 1'b1;
	repeat(4) @(posedge clk_i);
	enable_i = 1'b1;
	repeat(50) @(posedge clk_i);
	plaintext3_i ={16'h0000,16'h0000};
	plaintext2_i ={16'h0000,16'h0000};
	plaintext1_i ={16'h0000,16'h0000};
	plaintext0_i ={16'h0000,16'h8000};

end

endmodule
	
